unit Unit430;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, AdvEdit,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, AdvUtil, Vcl.Grids, AdvObj, BaseGrid,
  AdvGrid, Vcl.StdCtrls;

type


  // our own TEditLink
  // Copied exactly over.
  // MOST of the code here isn't needed for this demo
  // but left it in there anyways.

  TCMEditLink = class(TEditLink)
  Strict private
    FEdit: TAdvEdit;
  protected
    procedure EditExit(Sender: TObject);
  public
    procedure CreateEditor(aParent: TWinControl); override;
    procedure DestroyEditor; override;
    function GetEditorValue: string; override;
    procedure SetEditorValue(s: string); override;
    function GetEditControl: TWinControl; override;
    procedure SetProperties; override;
    // Use this to set MaxLength, CharCase and Type
    procedure QuickSet(Const MaxLength: Integer = 50;
      Const CharCase: teditcharcase = ecUpperCase;
      Const EditType: tadvedittype = etString);
    procedure SetStockNumber;
    procedure SetLocation;
    procedure SetDescription;
  published
    property Edit: TAdvEdit read FEdit write FEdit;
  end;

  TForm430 = class(TForm)
    grid: TAdvStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure GridGetEditorProp(Sender: TObject; ACol, ARow: Integer;
      AEditLink: TEditLink);
    procedure GridGetEditorType(Sender: TObject; ACol, ARow: Integer;
      var AEditor: TEditorType);
    procedure FormDestroy(Sender: TObject);
  private
    fCMEditor: TCMEditLink;
    procedure DropDownFix(Sender: TObject);
    procedure DropUpFix(Sender: TObject);

  public
    { Public declarations }
  end;

var
  Form430: TForm430;

implementation

{$R *.dfm}

procedure TForm430.FormCreate(Sender: TObject);
var
  x: Integer;
begin

  fCMEditor := TCMEditLink.Create(nil);
  // not 100% sure if this is good or not, but we've had it like this for years
  // where our edit object is global to the form itselfs life span

  grid.Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine,
    goRangeSelect, goEditing, goTabs];

  grid.MouseActions.DirectEdit := True;

  grid.RowCount := 20;
  grid.ColCount := 20;

  for x := 0 to grid.ColCount - 1 do
    grid.Cells[x, 0] := 'Col' + x.ToString;

  grid.RandomFill(false, 9999);

  // hide some columns
  grid.HideColumn(1);
  grid.HideColumn(3);

  // swap some collumns

  grid.SwapColumns(3, 8);
  grid.SwapColumns(4, 11);

  // i just copied over the settings from our original source
  // where this was all set

  grid.Navigation.AdvanceAuto := True;
  grid.Navigation.AutoComboDropSize := True;
  grid.MouseActions.DirectEdit := True;

  grid.Options := grid.Options - [goRangeSelect];
  grid.SortSettings.Show := True;
  grid.SortSettings.SingleColumn := false;
  grid.SortSettings.Full := True;
  grid.SortSettings.UndoSort := True;
  grid.ActiveRowShow := True;
  grid.FilterDropDownAuto := True;
  grid.FilterDropDownCheck := True;
  grid.FilterDropDown.CaseSensitive := false;
  grid.FilterDropDown.Height := 150;
  grid.FilterDropDown.AutoSize := True;
  grid.FilterDropDownMultiCol := True;
  grid.ControlLook.CheckAlwaysActive := True;
  grid.ControlLook.DropDownAlwaysVisible := True;
  grid.ControlLook.NoDisabledButtonLook := True;
  grid.ControlLook.NoDisabledCheckRadioLook := True;
  grid.DoubleBuffered := True;
  grid.HintShowCells := True;
  grid.HintShowLargeText := True;
  grid.EnableHTML := True;
end;

procedure TForm430.FormDestroy(Sender: TObject);
begin
  freeandnil(fCMEditor);
end;

procedure TForm430.GridGetEditorProp(Sender: TObject; ACol, ARow: Integer;
  AEditLink: TEditLink);
var
  c: char;
begin
  if ACol in [5, 10, 15] then
  begin
    grid.Combobox.OnDropDown := DropDownFix;
    grid.Combobox.OnDropUp := DropUpFix;
    grid.ClearComboString;
    grid.Combobox.items.Add('');
    for c := 'A' to 'Z' do
      grid.Combobox.items.Add(c);
    grid.SetComboSelectionString('A');
  end;
end;

procedure TForm430.GridGetEditorType(Sender: TObject; ACol, ARow: Integer;
  var AEditor: TEditorType);
begin
  if ACol in [5, 10, 15] then
    AEditor := edComboList
  else
  begin
    AEditor := edCustom;
    grid.EditLink := fCMEditor;
  end;
end;

procedure TForm430.DropDownFix(Sender: TObject);
begin
  SendMessage(self.Handle, WM_SETCURSOR, 0, 0);
  Screen.Cursor := crHandPoint;
end;

procedure TForm430.DropUpFix(Sender: TObject);
begin
  SendMessage(self.Handle, WM_SETCURSOR, 0, 0);
  Screen.Cursor := crDefault;
end;
/// /////////////////////////////////////////////////////////////////////////

procedure TCMEditLink.CreateEditor(aParent: TWinControl);
begin
  inherited;
  FEdit := TAdvEdit.Create(nil);
  FEdit.BorderStyle := bsNone;
  FEdit.OnKeydown := EditKeyDown;
  FEdit.OnExit := EditExit;
  FEdit.Width := 0;
  FEdit.Height := 0;
  FEdit.Parent := aParent;
  WantKeyLeftRight := True;
  WantKeyHomeEnd := True;
end;

procedure TCMEditLink.DestroyEditor;
begin
  if assigned(FEdit) then
    FEdit.Free;
  FEdit := nil;
end;

procedure TCMEditLink.SetProperties;
begin
  inherited;
  // This gets called FIRST before your Custom calls like SetStockNumber
  // This to be used to "Reset" the edit control.
  FEdit.MaxLength := 255;
  FEdit.CharCase := ecNormal;
  FEdit.EditType := etString;
  FEdit.InvalidChars := '';
end;

procedure TCMEditLink.QuickSet(Const MaxLength: Integer = 50;
  Const CharCase: teditcharcase = ecUpperCase;
  Const EditType: tadvedittype = etString);
begin
  FEdit.MaxLength := MaxLength;
  FEdit.CharCase := CharCase;
  FEdit.EditType := EditType;
end;

procedure TCMEditLink.SetStockNumber;
begin
  QuickSet(10, ecUpperCase, etInvalidChars);
  FEdit.InvalidChars := '! $%&''()*+./:;<=>?@[]^_`{}|~,#'
end;

procedure TCMEditLink.SetLocation;
begin
  QuickSet(10, ecUpperCase, etInvalidChars);
  FEdit.InvalidChars := '\},?*';
end;

procedure TCMEditLink.SetDescription;
begin
  QuickSet(256, ecNormal, etInvalidChars);
  FEdit.InvalidChars := '\}';
end;

function TCMEditLink.GetEditorValue: string;
begin
  Result := FEdit.Text;
end;

procedure TCMEditLink.SetEditorValue(s: string);
begin
  FEdit.Text := s;
end;

function TCMEditLink.GetEditControl: TWinControl;
begin
  Result := FEdit;
end;

procedure TCMEditLink.EditExit(Sender: TObject);
begin
  HideEditor;
end;

end.
